﻿namespace TabsStudioOrderEx
{
    public class OrderEx : TabsStudioExt.ITabsStudioAddin, TabsStudioExt.IConfigurable
    {
        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte)
        {
            engine = engine_;

            storage = new OptionsStorage(engine.GetAddinSettingsDirectory());
            options = storage.Load();
            
            engine.TabExtensionCreated += OnTabEvent;
        }

        public void OnDisconnection()
        {
            engine.TabExtensionCreated -= OnTabEvent;
        }

        public void Configure(System.Windows.Window parent)
        {
            OptionsDialog dialog = new OptionsDialog(options);
            dialog.Owner = parent;
            dialog.WindowStartupLocation = System.Windows.WindowStartupLocation.CenterOwner;
            if (dialog.ShowDialog().GetValueOrDefault())
                storage.Save(options);
        }

        private void OnTabEvent(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            OrderExtensions(e.Tab);
        }

        private void OrderExtensions(TabsStudioExt.Tab tab)
        {
            if (tab.TabExtensions.Count < 2)
                return;

            System.Windows.Controls.DockPanel dock = tab.TabItem.Header as System.Windows.Controls.DockPanel;
            if (dock == null || dock.Children.Count != tab.TabExtensions.Count + 3) // TabExtensionGroup(s) + TabNameGroup, OpenPreviewTabButton, CloseTabButton 
                return;

            for (int i = 1; i < tab.TabExtensions.Count; ++i)
            {
                System.Windows.UIElement extensionGroupLHS = dock.Children[1 + (i - 1)];
                System.Windows.UIElement extensionGroupRHS = dock.Children[1 + i];
                if (Priority(extensionGroupLHS) < Priority(extensionGroupRHS))
                {
                    dock.Children.RemoveAt(1 + i);
                    dock.Children.Insert(1 + (i - 1), extensionGroupRHS);
                }
            }
        }

        private int Priority(System.Windows.UIElement extensionGroup)
        {
            string ext = ((extensionGroup as System.Windows.Controls.StackPanel).Children[0] as 
                System.Windows.Controls.Label).Content as string;

            string[] priorities = options.extensionsPriority.Split(',');
            for (int i = 0; i < priorities.Length; ++i)
            {
                if (priorities[i].Equals(ext))
                    return priorities.Length - i;
            }

            return 0;
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private OptionsStorage storage;
        private Options options;
    }
}
